/*
 * glyph.c
 */

#include "init.h"
#include "face.h"
#include "glyph.h"
#include "gmetrics.h"
#include "outline.h"
#include "raster.h"
#include "error.h"
#include "debug.h"

static void
tt_free_glyph(glyph)
	TT_Glyph *glyph;
{
	Debug("free glyph begin");
	/* if not done */
	if (inited && glyph->z) {
		TT_Error error;

		error = TT_Done_Glyph(*glyph);
		if (error) tt_error_raise(error);
		glyph->z = NULL; /* for done again */
	}
	Debug("free glyph end");
}

static VALUE
tt_glyph_new(self, face_)
	VALUE self, face_;
{
	TT_Glyph *glyph;
	TT_Face *face;
	VALUE obj;
	TT_Error error;

	Data_Get_Struct(face_, TT_Face, face);
	obj = Data_Make_Struct(self, TT_Glyph, 0, tt_free_glyph, glyph);
	error = TT_New_Glyph(*face, glyph);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_glyph_done(self)
	VALUE self;
{
	TT_Glyph *glyph;
	TT_Error error;

	Data_Get_Struct(self, TT_Glyph, glyph);
	error = TT_Done_Glyph(*glyph);
	if (error) tt_error_raise(error);
	glyph->z = NULL; /* for done again */

	return Qnil;
}

static VALUE
tt_load_glyph(self, instance_, glyph_, index)
{
	TT_Instance *instance;
	TT_Glyph *glyph;
	TT_Error error;

	Data_Get_Struct(instance_, TT_Instance, instance);
	Data_Get_Struct(glyph_, TT_Glyph, glyph);
	error = TT_Load_Glyph(*instance, *glyph, FIX2INT(index), TTLOAD_DEFAULT);
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_glyph_get_outline(self)
	VALUE self;
{
	TT_Glyph *glyph;
	TT_Outline *outline;
	VALUE obj;
	TT_Error error;

	Data_Get_Struct(self, TT_Glyph, glyph);
	obj = Data_Make_Struct(cTT_outline, TT_Outline, 0, 0, outline);  /* owned */
	error = TT_Get_Glyph_Outline(*glyph, outline);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_glyph_get_metrics(self)
	VALUE self;
{
	TT_Glyph *glyph;
	TT_Glyph_Metrics gmetrics;
	VALUE obj;
	TT_Error error;

	Data_Get_Struct(self, TT_Glyph, glyph);
	error = TT_Get_Glyph_Metrics(*glyph, &gmetrics);
	if (error) tt_error_raise(error);
	obj = tt_gmetrics_setup(&gmetrics);

	return obj;
}

static VALUE
tt_glyph_get_bitmap(self, map_, x, y)
	VALUE self, map_, x, y;
{
	TT_Glyph *glyph;
	TT_Raster_Map *map;
	TT_Error error;

	Data_Get_Struct(self, TT_Glyph, glyph);
	Data_Get_Struct(map_, TT_Raster_Map, map);
	if (!map->size) {
		map->size = map->rows * map->cols;
		map->bitmap = ALLOC_N(char, map->size);
	}
	error = TT_Get_Glyph_Bitmap(*glyph, map, FIX2INT(x), FIX2INT(y));
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_glyph_get_pixmap(self, map_, x, y)
	VALUE self, map_, x, y;
{
	TT_Glyph *glyph;
	TT_Raster_Map *map;
	TT_Error error;

	Data_Get_Struct(self, TT_Glyph, glyph);
	Data_Get_Struct(map_, TT_Raster_Map, map);
	if (!map->size) {
		Debug("glyph new pixmap");
		map->size = map->rows * map->cols;
		map->bitmap = ALLOC_N(char, map->size);
	}
	error = TT_Get_Glyph_Pixmap(*glyph, map, FIX2INT(x), FIX2INT(y));
	if (error) tt_error_raise(error);

	return Qnil;
}

void
tt_init_glyph()
{
	/*
	 * glyph management
	 */
	cTT_glyph = rb_define_class_under(mTT, "Glyph", rb_cObject);

	rb_define_singleton_method(cTT_glyph, "new", tt_glyph_new, 1);
	rb_define_method(cTT_glyph, "done", tt_glyph_done, 0);
	rb_define_module_function(mTT, "load_glyph", tt_load_glyph, 3);
	rb_define_method(cTT_glyph, "outline", tt_glyph_get_outline, 0);
	rb_define_method(cTT_glyph, "metrics", tt_glyph_get_metrics, 0);
/*
	rb_define_method(cTT_glyph, "big_metrics", tt_glyph_get_big_metrics, 0);
*/
	rb_define_method(cTT_glyph, "get_bitmap", tt_glyph_get_bitmap, 3);
	rb_define_method(cTT_glyph, "get_pixmap", tt_glyph_get_pixmap, 3);
}
